import { MeshBVH } from "three-mesh-bvh";
import { Mesh, Ray, SkinnedMesh, Vector3 } from 'three';
export class BVHStorage {
    meshes = [];
    riggedMeshes = [];
    constructor(meshes) {
        for (const mesh of meshes) {
            if (!(mesh instanceof Mesh))
                continue;
            const bvh = new MeshBVH(mesh.geometry);
            const meshStore = {
                mesh: mesh,
                bvh
            };
            if (mesh instanceof SkinnedMesh)
                this.riggedMeshes.push(meshStore);
            else
                this.meshes.push(meshStore);
        }
        ;
    }
    ;
    updateRiggedMeshes() {
        for (var i = 0; i != this.riggedMeshes.length; i++)
            this.riggedMeshes[i].bvh = new MeshBVH(this.riggedMeshes[i].mesh.geometry);
    }
    ;
    updateMeshes() {
        for (var i = 0; i != this.meshes.length; i++)
            this.meshes[i].bvh = new MeshBVH(this.meshes[i].mesh.geometry);
    }
    ;
    raycast(origin, direction) {
        var currentObject = null;
        var currentObjectDistance = Infinity;
        for (const mesh of [...this.meshes, ...this.riggedMeshes]) {
            const point = mesh.bvh.raycastFirst(new Ray(origin, direction));
            if (currentObjectDistance > point.distance) {
                currentObject = mesh.mesh;
                currentObjectDistance = point.distance;
            }
            ;
        }
        ;
        return currentObject;
    }
    ;
    cameraRaycast(camera, coord) {
        const origin = camera.getWorldPosition(new Vector3(0, 0, 0));
        const direction = new Vector3(coord.x, coord.y, 0.5).unproject(camera).sub(origin).normalize();
        return this.raycast(origin, direction);
    }
    ;
}
;
