import { Group, WebGLRenderer } from "three";
import { OcclusionRaytracer } from "./occlusionRaytracer.js";
import { meshesOfScene } from './utils.js';
import { BVHStorage } from "./bvhStorage.js";
export class WebGLOcclusionRenderer extends WebGLRenderer {
    occlusionRaytracer;
    scene;
    /**
     * @description Update Delay in frames.
    */
    updateDelay = 30;
    /**
     * @description Current amount of frames rendered. Is reset when above the update delay.
     */
    updateDelayCurrentFrames = 0;
    currentUnoccludedMeshes = [];
    camera;
    constructor(args = {}, scene, camera) {
        super(args);
        this.scene = scene;
        this.occlusionRaytracer = new OcclusionRaytracer(new BVHStorage([]), 240, 240, camera);
        const sceneMeshes = meshesOfScene(this.scene);
        this.occlusionRaytracer.bvhStorage = new BVHStorage(sceneMeshes);
        this.camera = camera;
    }
    ;
    render() {
        if (this.updateDelayCurrentFrames >= this.updateDelay) {
            this.updateDelayCurrentFrames = 0;
            const sceneMeshes = meshesOfScene(this.scene);
            this.occlusionRaytracer.bvhStorage = new BVHStorage(sceneMeshes);
            this.currentUnoccludedMeshes = this.occlusionRaytracer.runOcclusionQuery();
        }
        ;
        const placeboScene = new Group();
        placeboScene.add(...this.currentUnoccludedMeshes);
        placeboScene.updateMatrixWorld(true);
        placeboScene.updateWorldMatrix(false, true);
        WebGLRenderer.prototype.render.apply(this, [placeboScene, this.camera]);
        this.updateDelayCurrentFrames++;
    }
    ;
}
;
